@@ -215,6 +215,12 @@ void free_db_line(db_line* dl)
   free(dl->tiger);
   free(dl->filename);
 
+  dl->filename=NULL;
+  dl->md5=NULL;
+  dl->sha1=NULL;
+  dl->rmd160=NULL;
+  dl->tiger=NULL;
+
 }
 
 int compare_db(list* new)
@@ -249,8 +255,9 @@ int compare_db(list* new)
       }else {
 	/* This line was ok */
 	free_db_line(old);
+	free(old);
 	free_db_line((db_line*)r->data);
-	r->data=NULL;
+	free((db_line*)r->data);
 	l=list_delete_item(r);
       }
     }
