@@ -53,17 +53,25 @@ int db_init(int inout)
 
 db_line* db_readline(db_config* conf){
   db_line* s=NULL;
-  
+  int i=0;
+
   switch (conf->db_in_url->type) {
   case url_file : {
     /* Should set errno */
     /* Please FIXME */
     if (conf->db_in!=NULL) {
       char** ss=db_readline_file(conf);
-      /* Please FIXME */
-      /* Add conversion from char** -> db_line* */
       if (ss!=NULL){
 	s=db_char2line(ss,conf);
+
+	for(i=0;i<conf->db_in_size;i++){
+	  if(ss[i]!=NULL){
+	    free(ss[i]);
+	    ss[i]=NULL;
+	  }
+	}
+	free(ss);
+	
       }
     }
       
@@ -79,9 +87,6 @@ db_line* db_readline(db_config* conf){
   
 }
 
-/*
-  FIX ME! No errorhandling done :(
- */
 byte* base64tobyte(char* src,int len)
 {
   if(strcmp(src,"0")!=0){
@@ -193,6 +198,7 @@ time_t base64totime_t(char* s){
   if (b==NULL||strcmp(s,"0")==0) {
     
     /* Should we print error here? */
+    free(b);
     
     return 0;
   } else {
@@ -200,8 +206,10 @@ time_t base64totime_t(char* s){
     
     if (endp[0]!='\0') {
       error(0,"Error converting base64\n");
+      free(b);
       return 0;
     }
+    free(b);
     return t;
   }
   
@@ -229,33 +237,6 @@ long readoct(char* s,char* err){
 }
 
 
-/* Is this obsolete? */
-
-char** db_line2char(db_line* line,db_config* conf){
-  
-  char** s=(char**)malloc(sizeof(char*)*conf->db_out_size);
-  int i;
-  
-  for (i=0;i<conf->db_out_size;i++){
-    switch (conf->db_out_order[i]) {
-      
-    default : {
-      /* FIX ME! 10 */
-      s[i]=(char*)malloc(sizeof(char)*10);
-      if (s[i]==NULL) {
-	error(0,"Not enough memory to write to file");
-      } else {
-	sprintf(s[i],"%i",line->size);
-      }
-    }
-    
-    
-    }
-    
-  }
-}
-
-
 int db_writespec()
 {
   switch (conf->db_out_url->type) {
@@ -278,7 +259,6 @@ int db_writespec()
   return RETFAIL;
 }
 
-/* Please FIXME (palauttaa ihan mit sattuu.) */
 
 int db_writeline(db_line* line,db_config* conf){
 
