@@ -152,6 +152,11 @@ char** db_readline_file(db_config* conf){
   }
   
   s=(char**)malloc(sizeof(char*)*db_unknown);
+
+  /* We NEED this to avoid Bus errors on Suns */
+  for(i=0;i<db_unknown;i++){
+    s[i]=NULL;
+  }
   
   for(i=0;i<conf->db_in_size;i++){
     switch (r=db_scan()) {
@@ -270,6 +275,7 @@ int db_writeint(long i,FILE* file,int a)
 int db_write_byte_base64(byte*data,size_t len,FILE* file,int i )
 {
   char* tmpstr=NULL;
+  int retval=0;
   
   tmpstr=encode_base64(data,len);
   if(i){
@@ -277,7 +283,9 @@ int db_write_byte_base64(byte*data,size_t len,FILE* file,int i )
   }
 
   if(tmpstr){
-    return fprintf(file,tmpstr);
+    retval=fprintf(file,tmpstr);
+    free(tmpstr);
+    return retval;
   }else {
     return fprintf(file,"0");
   }
@@ -288,7 +296,8 @@ int db_write_time_base64(time_t i,FILE* file,int a)
 {
   static char* ptr=NULL;
   char* tmpstr=NULL;
-  
+  int retval=0;
+
   if (ptr==NULL) {
     ptr=(char*)malloc(sizeof(char)*TIMEBUFSIZE);
     if (ptr==NULL) {
@@ -298,26 +307,17 @@ int db_write_time_base64(time_t i,FILE* file,int a)
   }
   memset((void*)ptr,0,sizeof(char)*TIMEBUFSIZE);
 
-  /* There is a bug in egcs-2.91.66 and you cannot call funktion which
-   * expexts 64-bit value with 32-bit value
-   * If anyone is intrested... there is an other bug also. I think
-   * it is sparc spesific. When calling function which returns structure
-   * calling sequense is invalid. Returnvalue is used when no space is
-   * reserved for it. Funny what you notice while doing a compiler by
-   * youself. 
-   */
-
-  /*
-    tmpstr=(char*)lltostr((long long)i,ptr);
-   */
-
   sprintf(ptr,"%i",i);
 
   if(a){
     fprintf(file," ");
   }
-  
-  return fprintf(file,encode_base64(ptr,strlen(ptr)));
+
+  tmpstr=encode_base64(ptr,strlen(ptr));
+  retval=fprintf(file,tmpstr);
+  free(tmpstr);
+  free(ptr);
+  return retval;
 
 }
 
