@@ -69,11 +69,19 @@ list* do_md(list* file_lst)
       bread=fread(buf,1,BUFSIZE,fh);
       md_write(mdh,(byte*)buf,bread);
     }
-    while(!feof(fh));
+    while(!feof(fh) && bread>0);
+    if(bread==0 && !feof(fh)){
+      error(5,"read error while reading %s\n", dl->filename);
+      dl->md5=NULL;
+      dl->sha1=NULL;
+      dl->tiger=NULL;
+      dl->rmd160=NULL;
+      fclose(fh);
+      continue;
+    }
     if(fclose(fh)){
       error(5,"do_md():fclose() for %s failed:%s\n",
 	    dl->filename,strerror(errno));
-      continue;
     }
     md_final(mdh); /* Let's flush the buffers */
 
@@ -102,5 +110,6 @@ list* do_md(list* file_lst)
 
   }
 
+  md_close(mdh);
   return file_lst;
 }
