@@ -72,8 +72,14 @@ void error(int errorlevel,char* error_msg,...)
   int retval=0;
   list* r=NULL;
 
-  if(conf->verbose_level<errorlevel){
-    return;
+  if(conf->verbose_level==-1){
+    if(5<errorlevel){
+      return;
+    }
+  }else{ 
+    if(conf->verbose_level<errorlevel){
+      return;
+    }
   }
 
   if(conf->report_fd==NULL){
