@@ -109,18 +109,15 @@ seltree* new_seltree_node(seltree* tree,char*path)
   return node;
 }
 
-seltree* gen_seltree(list* prxlist,list* nrxlist,list* erxlist)
-{
-  regex_t* rxtmp=NULL;
-  seltree* tree=NULL;
-  seltree* curnode=NULL;
-  char* rxtok=NULL;
-  list* r=NULL;
-  rx_rule* rxc=NULL;
-  
-  tree=new_seltree_node(NULL,"");
+void gen_seltree(list* rxlist,seltree* tree,char type){
 
-  for(r=prxlist;r;r=r->next){
+  regex_t* rxtmp=NULL;
+  seltree* curnode=NULL;
+  list* r=NULL;
+  char* rxtok=NULL;
+  rx_rule* rxc=NULL;
+
+  for(r=rxlist;r;r=r->next){
     rxtok=strrxtok(((rx_rule*)r->data)->rx);
     curnode=get_seltree_node(tree,rxtok);
     if(curnode==NULL){
@@ -134,49 +131,27 @@ seltree* gen_seltree(list* prxlist,list* nrxlist,list* erxlist)
       rxc->rx=((rx_rule*)r->data)->rx;
       rxc->crx=rxtmp;
       rxc->attr=((rx_rule*)r->data)->attr;
-      curnode->sel_rx_lst=list_append(curnode->sel_rx_lst,(void*)rxc);
+      switch (type){
+      case 's':{
+	curnode->sel_rx_lst=list_append(curnode->sel_rx_lst,(void*)rxc);
+	break;
+      }
+      case 'n':{
+	curnode->neg_rx_lst=list_append(curnode->neg_rx_lst,(void*)rxc);
+	break;
+      }
+      case 'e':{
+	curnode->equ_rx_lst=list_append(curnode->equ_rx_lst,(void*)rxc);
+	break;
+      }
+      }
     }
   }
 
-  for(r=nrxlist;r;r=r->next){
-    rxtok=strrxtok(((rx_rule*)r->data)->rx);
-    curnode=get_seltree_node(tree,rxtok);
-    if(curnode==NULL){
-      curnode=new_seltree_node(tree,rxtok);
-    }
-    rxtmp=(regex_t*)malloc(sizeof(regex_t));
-    if( regcomp(rxtmp,((rx_rule*)r->data)->rx,REG_EXTENDED|REG_NOSUB)){
-      error(0,"Error in negative regexp:%s",((rx_rule*)r->data)->rx);
-    }else{
-      rxc=(rx_rule*)malloc(sizeof(rx_rule));
-      rxc->rx=((rx_rule*)r->data)->rx;
-      rxc->crx=rxtmp;
-      rxc->attr=((rx_rule*)r->data)->attr;
-      curnode->neg_rx_lst=list_append(curnode->neg_rx_lst,(void*)rxc);
-    }
-  }
-  for(r=erxlist;r;r=r->next){
-    rxtok=strrxtok(((rx_rule*)r->data)->rx);
-    curnode=get_seltree_node(tree,rxtok);
-    if(curnode==NULL){
-      curnode=new_seltree_node(tree,rxtok);
-    }
-    rxtmp=(regex_t*)malloc(sizeof(regex_t));
-    if( regcomp(rxtmp,((rx_rule*)r->data)->rx,REG_EXTENDED|REG_NOSUB)){
-      error(0,"Error in equals regexp:%s",((rx_rule*)r->data)->rx);
-    }else{
-      rxc=(rx_rule*)malloc(sizeof(rx_rule));
-      rxc->rx=((rx_rule*)r->data)->rx;
-      rxc->crx=rxtmp;
-      rxc->attr=((rx_rule*)r->data)->attr;
-      curnode->equ_rx_lst=list_append(curnode->equ_rx_lst,(void*)rxc);
-    }
-  }
-  
-  return tree;
+
 }
 
-list* add_file_to_list(list* listp,char*filename,int attr)
+list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
 {
   db_line* fil=NULL;
   struct stat fs;
@@ -189,9 +164,12 @@ list* add_file_to_list(list* listp,char*filename,int attr)
   fil->filename=(char*)malloc(sizeof(char)*strlen(filename)+1);
   strncpy(fil->filename,filename,strlen(filename));
   fil->filename[strlen(filename)]='\0';
-  sres=stat(fil->filename,&fs);
+  /* We want to use lstat here instead of stat since we want *
+   * symlinks stats not the file that it points to. */
+  sres=lstat(fil->filename,&fs);
   if(sres==-1){
-    error(0,"stat() failed for %s:%s\n",fil->filename,strerror(errno));
+    error(0,"lstat() failed for %s:%s\n",fil->filename,strerror(errno));
+    *addok=RETFAIL;
     return listp;
   }
   if(DB_INODE&attr)
@@ -240,7 +218,8 @@ list* add_file_to_list(list* listp,char*filename,int attr)
     fil->atime=0;
 
   if(S_ISDIR(fs.st_mode)||S_ISCHR(fs.st_mode)
-     ||S_ISBLK(fs.st_mode)||S_ISFIFO(fs.st_mode)){
+     ||S_ISBLK(fs.st_mode)||S_ISFIFO(fs.st_mode)
+     ||S_ISLNK(fs.st_mode)){
     fil->md5=NULL;
     fil->sha1=NULL;
     fil->tiger=NULL;
@@ -255,6 +234,7 @@ list* add_file_to_list(list* listp,char*filename,int attr)
   }
   listp=list_append(listp,(void*)fil);
 
+  *addok=RETOK;
   return listp;
 }
 
@@ -315,6 +295,7 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
   int e=0;
   int matchattr=0;
   long td=0;
+  int addok=RETOK;
 
   if( strncmp(tree->path,"",strlen(tree->path))==0)
     tree->path="/";
@@ -361,8 +342,8 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
 	/* We use attr here instead of matchattr since this dir has been 
 	 * selected from a higher level and we want to use that mask which
 	 * was used with that selection */
-	file_lst=add_file_to_list(file_lst,fullname,attr);
-	if(addfile!=2){
+	file_lst=add_file_to_list(file_lst,fullname,attr,&addok);
+	if(addfile!=2 && addok!=RETFAIL){
 	  if(S_ISDIR(((db_line*)file_lst->tail->data)->perm)){
 	    a=get_seltree_node(tree,((db_line*)file_lst->tail->data)->filename);
 	    if(a==NULL)
@@ -378,8 +359,8 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
       
       if(addfile){
 	file_lst=add_file_to_list(file_lst,fullname,
-				  matchattr);
-	if(addfile!=2){
+				  matchattr,&addok);
+	if(addfile!=2 && addok!=RETFAIL){
 	  if(S_ISDIR(((db_line*)file_lst->tail->data)->perm)){
 	    a=get_seltree_node(tree,
 			       ((db_line*)file_lst->tail->data)->filename);
@@ -396,6 +377,10 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
     }
     
   }
+
+  if(closedir(dirh)==-1){
+    error(0,"Closedir() failed for %s\n",tree->path);
+  };
   
   /* All childs are not necessarily checked yet */
   if(tree->childs!=NULL){
@@ -414,8 +399,12 @@ list* gen_list(list* prxlist,list* nrxlist,list* erxlist)
   list* r=NULL;
   seltree* tree=NULL;
 
+  tree=new_seltree_node(NULL,"");
+
+  gen_seltree(prxlist,tree,'s');
+  gen_seltree(nrxlist,tree,'n');
+  gen_seltree(erxlist,tree,'e');
 
-  tree=gen_seltree(prxlist,nrxlist,erxlist);
   if(tree==NULL)
     return NULL;
   
